<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * External services used for AJAX call
 *
 * @package     quizaccess_quilgo
 * @copyright   2023 Native Platform Ltd <hello@quilgo.com>
 * @license     http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

defined('MOODLE_INTERNAL') || die();

use quizaccess_quilgo\plasmapi;

require_once($CFG->libdir.'/externallib.php');
require_once($CFG->dirroot. '/mod/quiz/accessrule/quilgo/lib.php');

class quizaccess_quilgo_external extends external_api {

    const REPORT_GENERATED_FAILED = 0;
    const REPORT_GENERATED_SUCCESS = 1;
    const REPORT_GENERATED_NOTREADY = 2;

    /**
     * Define parameters when store sessionid
     * @return mixed
     */
    public static function store_sessionid_parameters() {
        return new external_function_parameters(
            [
                'attemptid' => new external_value(PARAM_INT, 'Attempt id'),
                'plasmsessionid' => new external_value(PARAM_TEXT, 'Plasm session id'),
                'camera_enabled' => new external_value(PARAM_INT, 'Camera enabled'),
                'screen_enabled' => new external_value(PARAM_INT, 'Screen enabled'),
                'force_enabled' => new external_value(PARAM_INT, 'Force enabled'),
                'cmid' => new external_value(PARAM_INT, 'Course module id'),
            ]
        );
    }

    /**
     * Define returns structure after store sessionid
     * @return mixed
     */
    public static function store_sessionid_returns() {
        return new external_single_structure([
            'attemptid' => new external_value(PARAM_NOTAGS, 'Attempt id'),
        ]);
    }

    /**
     * Handle store sessionid
     * @param int $attemptid Attemptid of the session
     * @param int $plasmsessionid The plasm session id
     * @param int $cameraenabled Flag for camera tracking enabled or not
     * @param int $screenenabled Flag for screen tracking enabled or not
     * @param int $forceenabled Flag for force tracking enabled or not
     * @param int $cmid course module id
     * @return array
     */
    public static function store_sessionid($attemptid, $plasmsessionid, $cameraenabled, $screenenabled, $forceenabled, $cmid) {
        global $DB;

        $params = [
            'attemptid' => $attemptid,
            'plasmsessionid' => $plasmsessionid,
            'camera_enabled' => $cameraenabled,
            'screen_enabled' => $screenenabled,
            'force_enabled' => $forceenabled,
            'cmid' => $cmid,
        ];

        self::validate_parameters(self::store_sessionid_parameters(), $params);

        $context = context_module::instance($cmid, MUST_EXIST);
        require_capability('quizaccess/quilgo:storesessionid', $context);

        $currentreport = $DB->get_record('quizaccess_quilgo_reports', ['attemptid' => $attemptid]);
        if (!$currentreport) {
            $DB->insert_record('quizaccess_quilgo_reports', $params);
        }

        return ['attemptid' => $attemptid];
    }


    /**
     * Define parameters when fetch report
     * @return mixed
     */
    public static function fetch_report_parameters() {
        return new external_function_parameters([
            'attemptid' => new external_value(PARAM_INT, 'Attempt id'),
            'cmid' => new external_value(PARAM_INT, 'Course module id'),
        ]);
    }

    /**
     * Define returns structure after fetch report
     * @return mixed
     */
    public static function fetch_report_returns() {
        return new external_single_structure([
            'reportGeneratedCode' => new external_value(PARAM_TEXT, 'Report generated'),
            'expiresIn' => new external_value(PARAM_INT, 'Report images expires in'),
            'uuid' => new external_value(PARAM_TEXT, 'Report UUID'),
            'stat' => new external_single_structure(
                [
                    'framesTotal' => new external_value(PARAM_INT, 'Frames total'),
                    'facesPresence' => new external_value(PARAM_NUMBER, 'Face presense total'),
                    'avgFacesPerFrame' => new external_value(PARAM_NUMBER, 'Average faces per frame'),
                    'maxFacesPerFrame' => new external_value(PARAM_NUMBER, 'Max faces per frame'),
                    'pageUnfocusedCount' => new external_value(PARAM_INT, 'Page unfocused total'),
                ],
                'Statistic section',
                VALUE_OPTIONAL,
                null,
            ),
            'isStatDelivered' => new external_value(PARAM_BOOL, 'Stats delivered status'),
            'isSnapshotsLimitReached' => new external_value(PARAM_BOOL, 'Snapshots limit reached status'),
            'isScreenshotsLimitReached' => new external_value(PARAM_BOOL, 'Screenshots limit reached status'),
            'imageExpirationSeconds' => new external_value(PARAM_INT, 'Image expiration'),
            'snapshots' => new external_multiple_structure(
                new external_single_structure([
                    'id' => new external_value(PARAM_ALPHANUM, 'Snapshot id'),
                    'createdAt' => new external_value(PARAM_TEXT, 'Snapshot created at'),
                    'url' => new external_value(PARAM_URL, 'Snapshot url'),
                    'thumbnail' => new external_value(PARAM_URL, 'Snapshot thumbnail url'),
                    'facesAmount' => new external_value(PARAM_INT, 'Faces amount inside snapshot'),
                ]),
                'Snapshots content',
                VALUE_OPTIONAL,
                null,
            ),
            'screenshots' => new external_multiple_structure(
                new external_single_structure([
                    'id' => new external_value(PARAM_ALPHANUM, 'Screenshot id'),
                    'createdAt' => new external_value(PARAM_TEXT, 'Screenshot created at'),
                    'url' => new external_value(PARAM_URL, 'Screenshot url'),
                    'thumbnail' => new external_value(PARAM_URL, 'Screenshot thumbnail url'),
                    'isPageUnfocused' => new external_value(PARAM_BOOL, 'Page focused status inside screenshot'),
                ]),
                'Screenshots content',
                VALUE_OPTIONAL,
                null,
            ),
            'cameraEnabled' => new external_value(PARAM_INT, 'Report camera tracking enabled status'),
            'screenEnabled' => new external_value(PARAM_INT, 'Report screen tracking enabled status'),
        ]);
    }

    /**
     * Handle fetch report
     * @param int $attemptid Attempt id of the report
     * @param int $cmid Course module id of the report
     * @return mixed
     */
    public static function fetch_report($attemptid, $cmid) {
        global $DB;

        $params = ['attemptid' => $attemptid, 'cmid' => $cmid];
        self::validate_parameters(self::fetch_report_parameters(), $params);

        $context = context_module::instance($cmid, MUST_EXIST);
        require_capability('quizaccess/quilgo:fetchreport', $context);

        $responsestat = new stdClass();
        $responsestat->framesTotal = 0;
        $responsestat->facesPresence = 0;
        $responsestat->avgFacesPerFrame = 0;
        $responsestat->maxFacesPerFrame = 0;
        $responsestat->pageUnfocusedCount = 0;

        $response = new stdClass();
        $response->reportGeneratedCode = self::REPORT_GENERATED_FAILED;
        $response->expiresIn = 0;
        $response->uuid = null;
        $response->stat = $responsestat;
        $response->isStatDelivered = false;
        $response->isSnapshotsLimitReached = false;
        $response->isScreenshotsLimitReached = false;
        $response->imageExpirationSeconds = 0;
        $response->snapshots = [];
        $response->screenshots = [];
        $response->cameraEnabled = 0;
        $response->screenEnabled = 0;

        $attemptsession = get_attempt_session($attemptid);
        if (empty($attemptsession)) {
            return $response;
        }

        $plasmapi = new plasmapi();
        $report = $plasmapi->fetch_report($attemptsession->plasmsessionid);
        if (empty($report)) {
            return $response;
        }

        $attempt = get_attempt($attemptid);

        // Now we will treat Preview Attempt RESOURCE_NOT_FOUND as report not ready yet.
        // Because current condition we not attach webhook to plugin.
        // So we cannot store the generated stat from Plasm.
        if (!empty($report->error) && $report->error->code === "RESOURCE_NOT_FOUND") {
            if (!empty($attempt) && $attempt->preview == 1) {
                $response->reportGeneratedCode = self::REPORT_GENERATED_NOTREADY;
            }

            return $response;
        }

        if (!empty($attempt)) {
            $response->expiresIn = build_report_expires($attempt->timestart);
        }

        $response->reportGeneratedCode = self::REPORT_GENERATED_SUCCESS;
        $response->uuid = $report->uuid;
        $response->isStatDelivered = $report->isStatDelivered;
        $response->isSnapshotsLimitReached = $report->isSnapshotsLimitReached;
        $response->isScreenshotsLimitReached = $report->isScreenshotsLimitReached;
        $response->imageExpirationSeconds = $report->imageExpirationSeconds;
        $response->cameraEnabled = $attemptsession->camera_enabled;
        $response->screenEnabled = $attemptsession->screen_enabled;

        if (!empty($report->stat)) {
            $reportstat = $report->stat;

            $stat = new stdClass();
            $stat->framesTotal = $reportstat->framesTotal ?? null;
            $stat->facesPresence = $reportstat->facesPresence ?? null;
            $stat->avgFacesPerFrame = $reportstat->avgFacesPerFrame ?? null;
            $stat->maxFacesPerFrame = $reportstat->maxFacesPerFrame ?? null;
            $stat->pageUnfocusedCount = $reportstat->pageUnfocusedCount ?? null;

            $response->stat = $stat;
        }

        if (!empty($report->snapshots)) {
            $reportsnapshots = $report->snapshots;

            $snapshots = [];
            foreach ($reportsnapshots as $row) {
                $snapshot = new stdClass();
                $snapshot->id = $row->id;
                $snapshot->createdAt = $row->createdAt;
                $snapshot->url = $row->url;
                $snapshot->thumbnail = $row->thumbnail;
                $snapshot->facesAmount = $row->facesAmount;

                $snapshots[] = $snapshot;
            }

            $response->snapshots = $snapshots;
        }

        if (!empty($report->screenshots)) {
            $reportscreenshots = $report->screenshots;

            $screenshots = [];
            foreach ($reportscreenshots as $row) {
                $screenshot = new stdClass();
                $screenshot->id = $row->id;
                $screenshot->createdAt = $row->createdAt;
                $screenshot->url = $row->url;
                $screenshot->thumbnail = $row->thumbnail;
                $screenshot->isPageUnfocused = $row->isPageUnfocused;

                $screenshots[] = $screenshot;
            }

            $response->screenshots = $screenshots;
        }

        return $response;
    }
}
