<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Define some reusable logic or constants
 * @package     quizaccess_quilgo
 * @copyright   2023 Native Platform Ltd <hello@quilgo.com>
 * @license     http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

const PLASM_URL = 'https://moodle.integrations.quilgo.com';
const PLASM_MAX_SHOTS_PER_QUIZ = 30;
const PLASM_REPORTS_EXPIRE_IN_DAYS = 10;

const CONFIG_API_TOKEN_NAME = 'api_token';
const CONFIG_CLIENT_TOKEN_NAME = 'client_token';

const REPORT_FILE_PATH = '/mod/quiz/accessrule/quilgo/report.php';
const API_SETTINGS_PATH = '/mod/quiz/accessrule/quilgo/api_config.php';

function is_plugin_configured() {
    $apitoken = get_config('quizaccess_quilgo', CONFIG_API_TOKEN_NAME);
    $clienttoken = get_config('quizaccess_quilgo', CONFIG_CLIENT_TOKEN_NAME);

    return !empty($apitoken) && !empty($clienttoken);
}

function get_plasm_lib_url() {
    return PLASM_URL . '/public/collector.min.js';
}

function get_plasm_config() {
    $plasmconfig = new stdClass();
    $plasmconfig->clientToken = get_config('quizaccess_quilgo', CONFIG_CLIENT_TOKEN_NAME);
    $plasmconfig->apiEndpoint = PLASM_URL . '/api/v1/&';
    $plasmconfig->maxShotsPerQuiz = PLASM_MAX_SHOTS_PER_QUIZ;

    return $plasmconfig;
}

function get_quiz_data($quiz, $plasmsessionid = null) {
    $quizdata = new stdClass();
    $quizdata->id = $quiz->id;
    $quizdata->plasm_screen = $quiz->plasm_screen ?? 0;
    $quizdata->plasm_camera = $quiz->plasm_camera ?? 0;
    $quizdata->plasm_force = $quiz->plasm_force ?? 0;
    $quizdata->plasm_sessionid = $plasmsessionid;
    $quizdata->timeclose = $quiz->timeclose;
    $quizdata->timelimit = $quiz->timelimit;

    return $quizdata;
}

function is_use_tracking($quiz) {
    $quizdata = get_quiz_data($quiz);
    $plasmcamera = $quizdata->plasm_camera ?? 0;
    $plasmscreen = $quizdata->plasm_screen ?? 0;

    return $plasmcamera == 1 || $plasmscreen == 1;
}

function get_report_url($cid, $cmid, $quizId) {
    $reportparameters = [
        'cid' => $cid,
        'cmid' => $cmid,
        'qid' => $quizId
    ];

    return new moodle_url(REPORT_FILE_PATH, $reportparameters);
}

function get_attempt($attemptid) {
    global $DB;

    $sql = "SELECT * FROM {quiz_attempts} WHERE id = :attemptid";
    $attempt = $DB->get_record_sql($sql, ['attemptid' => $attemptid]);

    return $attempt ? $attempt : null;
}

function get_attempt_session($attemptid) {
    global $DB;

    $sql = "SELECT * FROM {quizaccess_quilgo_reports} WHERE attemptid = :attemptid";
    $attemptreportrow = $DB->get_record_sql($sql, ['attemptid' => $attemptid]);

    return $attemptreportrow ? $attemptreportrow : null;
}

function show_error_log($string) {
    debugging("QUIZACCESS QUILGO ERROR");
    debugging($string);
}


function dd($data) {
    echo '<pre>';
    die(var_dump($data));
}

function build_report_expires($starttime) {
    $configexpiration = PLASM_REPORTS_EXPIRE_IN_DAYS;
    $expirestime = strtotime("+$configexpiration days", $starttime);
    $expiresindays = ceil(($expirestime - $starttime) / 60 / 60 / 24);

    return $expiresindays;
}

function register_site() {
    $plasmapi = new \quizaccess_quilgo\plasmapi();
    $resp = null;

    try {
        $resp = $plasmapi->register();
    } catch (\Throwable $th) {
        $resp = null;
    }

    $apitoken = '';
    $clienttoken = '';
    if (!empty($resp)) {
        $apitoken = $resp->apiToken;
        $clienttoken = $resp->clientToken;
    }

    set_config(CONFIG_API_TOKEN_NAME, $apitoken, 'quizaccess_quilgo');
    set_config(CONFIG_CLIENT_TOKEN_NAME, $clienttoken, 'quizaccess_quilgo');

    return $resp;
}
